@@ -3,54 +3,24 @@
 __author__ = "Carlos Moro <cfmoro1976@yahoo.es>, Karsten Hilbert <Karsten.Hilbert@gmx.net>"
 __license__ = 'GPL v2 or later (for details see http://gnu.org)'
 
-import sys, logging
+import sys
+import logging
 
 
 if __name__ == '__main__':
 	sys.path.insert(0, '../../')
-from Gnumed.pycommon import gmPG2, gmExceptions, gmBusinessDBObject, gmTools, gmDispatcher, gmHooks
+from Gnumed.pycommon import gmPG2
+from Gnumed.pycommon import gmBusinessDBObject
+from Gnumed.pycommon import gmTools
+from Gnumed.pycommon import gmDispatcher
+from Gnumed.pycommon import gmHooks
+
 from Gnumed.business import gmCoding
-
-
-try:
-	_('dummy-no-need-to-translate-but-make-epydoc-happy')
-except NameError:
-	_ = lambda x:x
+from Gnumed.business import gmSoapDefs
 
 
 _log = logging.getLogger('gm.emr')
 
-
-soap_cat2l10n = {
-	u's': _('soap_S').replace(u'soap_', u''),
-	u'o': _('soap_O').replace(u'soap_', u''),
-	u'a': _('soap_A').replace(u'soap_', u''),
-	u'p': _('soap_P').replace(u'soap_', u''),
-	u'u': _('soap_U').replace(u'soap_', u''),
-#	u'u': u'?',
-	None: gmTools.u_ellipsis,
-	u'': gmTools.u_ellipsis
-}
-
-soap_cat2l10n_str = {
-	u's': _('soap_Subjective').replace(u'soap_', u''),
-	u'o': _('soap_Objective').replace(u'soap_', u''),
-	u'a': _('soap_Assessment').replace(u'soap_', u''),
-	u'p': _('soap_Plan').replace(u'soap_', u''),
-	u'u': _('soap_Unspecified').replace(u'soap_', u''),
-	None: _('soap_Administrative').replace(u'soap_', u'')
-}
-
-l10n2soap_cat = {
-	_('soap_S').replace(u'soap_', u''): u's',
-	_('soap_O').replace(u'soap_', u''): u'o',
-	_('soap_A').replace(u'soap_', u''): u'a',
-	_('soap_P').replace(u'soap_', u''): u'p',
-	_('soap_U').replace(u'soap_', u''): u'u',
-#	u'?': u'u',
-	gmTools.u_ellipsis: None
-}
-
 #============================================================
 def _on_soap_modified():
 	"""Always relates to the active patient."""
@@ -60,8 +30,8 @@ gmDispatcher.connect(_on_soap_modified, u'clin.clin_narrative_mod_db')
 
 #============================================================
 class cNarrative(gmBusinessDBObject.cBusinessDBObject):
-	"""Represents one clinical free text entry.
-	"""
+	"""Represents one clinical free text entry."""
+
 	_cmd_fetch_payload = u"SELECT * FROM clin.v_narrative WHERE pk_narrative = %s"
 	_cmds_store_payload = [
 		u"""update clin.clin_narrative set
@@ -86,15 +56,18 @@ class cNarrative(gmBusinessDBObject.cBusinessDBObject):
 		'pk_encounter'
 	]
 
+	#--------------------------------------------------------
+	def format_maximum_information(self, patient=None):
+		return self.format(fancy = True, width = 70).split(u'\n')
+
 	#--------------------------------------------------------
 	def format(self, left_margin=u'', fancy=False, width=75):
 
 		if fancy:
-			# FIXME: add revision
 			txt = gmTools.wrap (
 				text = _('%s: %s by %.8s (v%s)\n%s') % (
 					self._payload[self._idx['date']].strftime('%x %H:%M'),
-					soap_cat2l10n_str[self._payload[self._idx['soap_cat']]],
+					gmSoapDefs.soap_cat2l10n_str[self._payload[self._idx['soap_cat']]],
 					self._payload[self._idx['modified_by']],
 					self._payload[self._idx['row_version']],
 					self._payload[self._idx['narrative']]
@@ -106,7 +79,7 @@ class cNarrative(gmBusinessDBObject.cBusinessDBObject):
 		else:
 			txt = u'%s [%s]: %s (%.8s)' % (
 				self._payload[self._idx['date']].strftime('%x %H:%M'),
-				soap_cat2l10n[self._payload[self._idx['soap_cat']]],
+				gmSoapDefs.soap_cat2l10n[self._payload[self._idx['soap_cat']]],
 				self._payload[self._idx['narrative']],
 				self._payload[self._idx['modified_by']]
 			)
@@ -114,6 +87,7 @@ class cNarrative(gmBusinessDBObject.cBusinessDBObject):
 				txt = txt[:width] + gmTools.u_ellipsis
 
 		return txt
+
 	#--------------------------------------------------------
 	def add_code(self, pk_code=None):
 		"""<pk_code> must be a value from ref.coding_system_root.pk_coding_system (clin.lnk_code2item_root.fk_generic_code)"""
@@ -140,6 +114,7 @@ class cNarrative(gmBusinessDBObject.cBusinessDBObject):
 		}
 		rows, idx = gmPG2.run_rw_queries(queries = [{'cmd': cmd, 'args': args}])
 		return
+
 	#--------------------------------------------------------
 	def remove_code(self, pk_code=None):
 		"""<pk_code> must be a value from ref.coding_system_root.pk_coding_system (clin.lnk_code2item_root.fk_generic_code)"""
@@ -150,6 +125,7 @@ class cNarrative(gmBusinessDBObject.cBusinessDBObject):
 		}
 		rows, idx = gmPG2.run_rw_queries(queries = [{'cmd': cmd, 'args': args}])
 		return True
+
 	#--------------------------------------------------------
 	# properties
 	#--------------------------------------------------------
@@ -189,23 +165,58 @@ class cNarrative(gmBusinessDBObject.cBusinessDBObject):
 		return
 
 	generic_codes = property(_get_generic_codes, _set_generic_codes)
+
 #============================================================
-# convenience functions
+def create_progress_note(soap=None, episode_id=None, encounter_id=None, link_obj=None):
+	"""Create clinical narrative entries.
+
+	<soap>
+		must be a dict, the keys being SOAP categories (including U and
+		None=admin) and the values being text (possibly multi-line)
+
+	Existing but empty ('' or None) categories are skipped.
+	"""
+	if soap is None:
+		return True
+
+	if link_obj is None:
+		link_obj = gmPG2.get_connection(readonly = False)
+		conn_rollback = link_obj.rollback
+		conn_commit = link_obj.commit
+		conn_close = link_obj.close
+	else:
+		conn_rollback = lambda x:x
+		conn_commit = lambda x:x
+		conn_close = lambda x:x
+
+	instances = {}
+	for cat in soap:
+		if cat not in gmSoapDefs.KNOWN_SOAP_CATS:
+			conn_rollback()
+			conn_close()
+			raise ValueError(u'invalid SOAP category [%s] in <soap> dictionary: %s', cat, soap)
+		val = soap[cat]
+		if val is None:
+			continue
+		if u''.join([ v.strip() for v in val ]) == u'':
+			continue
+		instance = create_narrative_item (
+			narrative = u'\n'.join([ v.strip() for v in val ]),
+			soap_cat = cat,
+			episode_id = episode_id,
+			encounter_id = encounter_id,
+			link_obj = link_obj
+		)
+		if instance is None:
+			continue
+		instances[cat] = instance
+
+	conn_commit()
+	conn_close()
+	return instances
+
 #============================================================
-def search_text_across_emrs(search_term=None):
-
-	if search_term is None:
-		return []
-
-	if search_term.strip() == u'':
-		return []
-
-	cmd = u'select * from clin.v_narrative4search where narrative ~* %(term)s order by pk_patient limit 1000'
-	rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': {'term': search_term}}], get_col_idx = False)
-
-	return rows
-#============================================================
-def create_clin_narrative(narrative=None, soap_cat=None, episode_id=None, encounter_id=None, link_obj=None):
+def create_narrative_item(narrative=None, soap_cat=None, episode_id=None, encounter_id=None, link_obj=None):
 	"""Creates a new clinical narrative entry
 
 		narrative - free text clinical narrative
@@ -220,7 +231,7 @@ def create_clin_narrative(narrative=None, soap_cat=None, episode_id=None, encoun
 	# 1) silently do not insert empty narrative
 	narrative = narrative.strip()
 	if narrative == u'':
-		return (True, None)
+		return None
 
 	# 2) also, silently do not insert true duplicates
 	# FIXME: this should check for .provider = current_user but
@@ -245,7 +256,7 @@ def create_clin_narrative(narrative=None, soap_cat=None, episode_id=None, encoun
 	rows, idx = gmPG2.run_ro_queries(link_obj = link_obj, queries = [{'cmd': cmd, 'args': args}], get_col_idx = True)
 	if len(rows) == 1:
 		narrative = cNarrative(row = {'pk_field': 'pk_narrative', 'data': rows[0], 'idx': idx})
-		return (True, narrative)
+		return narrative
 
 	# insert new narrative
 	queries = [
@@ -265,13 +276,15 @@ def create_clin_narrative(narrative=None, soap_cat=None, episode_id=None, encoun
 	rows, idx = gmPG2.run_rw_queries(link_obj = link_obj, queries = queries, return_data = True, get_col_idx = True)
 
 	narrative = cNarrative(row = {'pk_field': 'pk_narrative', 'idx': idx, 'data': rows[0]})
-	return (True, narrative)
+	return narrative
+
 #------------------------------------------------------------
 def delete_clin_narrative(narrative=None):
 	"""Deletes a clin.clin_narrative row by it's PK."""
 	cmd = u"delete from clin.clin_narrative where pk=%s"
 	rows, idx = gmPG2.run_rw_queries(queries = [{'cmd': cmd, 'args': [narrative]}])
 	return True
+
 #------------------------------------------------------------
 def get_narrative(since=None, until=None, encounters=None, episodes=None, issues=None, soap_cats=None, providers=None, patient=None, order_by=None):
 	"""Get SOAP notes pertinent to this encounter.
@@ -367,7 +380,7 @@ def get_as_journal(since=None, until=None, encounters=None, episodes=None, issue
 		raise ValueError('at least one of <patient>, <episodes>, <issues>, <encounters> must not be None')
 
 	if order_by is None:
-		order_by = u'ORDER BY vemrj.clin_when, vemrj.pk_episode, scr, vemrj.src_table'
+		order_by = u'ORDER BY c_vej.clin_when, c_vej.pk_episode, scr, c_vej.modified_when, c_vej.src_table'
 	else:
 		order_by = u'ORDER BY %s' % order_by
 
@@ -375,50 +388,60 @@ def get_as_journal(since=None, until=None, encounters=None, episodes=None, issue
 	args = {}
 
 	if patient is not None:
-		where_parts.append(u'pk_patient = %(pat)s')
+		where_parts.append(u'c_vej.pk_patient = %(pat)s')
 		args['pat'] = patient
 
 	if soap_cats is not None:
 		# work around bug in psycopg2 not being able to properly
 		# adapt None to NULL inside tuples
 		if None in soap_cats:
-			where_parts.append(u'((vemrj.soap_cat IN %(soap_cat)s) OR (vemrj.soap_cat IS NULL))')
+			where_parts.append(u'((c_vej.soap_cat IN %(soap_cat)s) OR (c_vej.soap_cat IS NULL))')
 			soap_cats.remove(None)
 		else:
-			where_parts.append(u'vemrj.soap_cat IN %(soap_cat)s')
+			where_parts.append(u'c_vej.soap_cat IN %(soap_cat)s')
 		args['soap_cat'] = tuple(soap_cats)
 
 	if time_range is not None:
-		where_parts.append(u"vemrj.clin_when > (now() - '%s days'::interval)" % time_range)
+		where_parts.append(u"c_vej.clin_when > (now() - '%s days'::interval)" % time_range)
 
 	if episodes is not None:
-		where_parts.append(u"vemrj.pk_episode IN %(epis)s")
+		where_parts.append(u"c_vej.pk_episode IN %(epis)s")
 		args['epis'] = tuple(episodes)
 
 	if issues is not None:
-		where_parts.append(u"vemrj.pk_health_issue IN %(issues)s")
+		where_parts.append(u"c_vej.pk_health_issue IN %(issues)s")
 		args['issues'] = tuple(issues)
 
 	# FIXME: implement more constraints
 
 	cmd = u"""
 		SELECT
-			to_char(vemrj.clin_when, 'YYYY-MM-DD') AS date,
-			vemrj.clin_when,
-			coalesce(vemrj.soap_cat, '') as soap_cat,
-			vemrj.narrative,
-			vemrj.src_table,
-
-			(SELECT rank FROM clin.soap_cat_ranks WHERE soap_cat = vemrj.soap_cat) AS scr,
-
-			vemrj.modified_when,
-			to_char(vemrj.modified_when, 'YYYY-MM-DD HH24:MI') AS date_modified,
-			vemrj.modified_by,
-			vemrj.row_version,
-			vemrj.pk_episode,
-			vemrj.pk_encounter,
-			vemrj.soap_cat as real_soap_cat
-		FROM clin.v_emr_journal vemrj
+			to_char(c_vej.clin_when, 'YYYY-MM-DD') AS date,
+			c_vej.clin_when,
+			coalesce(c_vej.soap_cat, '') as soap_cat,
+			c_vej.narrative,
+			c_vej.src_table,
+			c_scr.rank AS scr,
+			c_vej.modified_when,
+			to_char(c_vej.modified_when, 'YYYY-MM-DD HH24:MI') AS date_modified,
+			c_vej.modified_by,
+			c_vej.row_version,
+			c_vej.pk_episode,
+			c_vej.pk_encounter,
+			c_vej.soap_cat as real_soap_cat,
+			c_vej.src_pk,
+			c_vej.pk_health_issue,
+			c_vej.health_issue,
+			c_vej.episode,
+			c_vej.issue_active,
+			c_vej.issue_clinically_relevant,
+			c_vej.episode_open,
+			c_vej.encounter_started,
+			c_vej.encounter_last_affirmed,
+			c_vej.encounter_l10n_type
+		FROM
+			clin.v_emr_journal c_vej
+				join clin.soap_cat_ranks c_scr on (c_scr.soap_cat = c_vej.soap_cat)
 		WHERE
 			%s
 		%s""" % (
@@ -428,6 +451,23 @@ def get_as_journal(since=None, until=None, encounters=None, episodes=None, issue
 
 	rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': args}], get_col_idx = True)
 	return rows
+
+#============================================================
+# convenience functions
+#============================================================
+def search_text_across_emrs(search_term=None):
+
+	if search_term is None:
+		return []
+
+	if search_term.strip() == u'':
+		return []
+
+	cmd = u'select * from clin.v_narrative4search where narrative ~* %(term)s order by pk_patient limit 1000'
+	rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': {'term': search_term}}], get_col_idx = False)
+
+	return rows
+
 #============================================================
 # main
 #------------------------------------------------------------
@@ -458,7 +498,7 @@ if __name__ == '__main__':
 		#print "codes:", diagnose.get_codes()
 
 		#print "creating narrative..."
-		#status, new_narrative = create_clin_narrative(narrative = 'Test narrative', soap_cat = 'a', episode_id=1, encounter_id=2)
+		#new_narrative = create_narrative_item(narrative = 'Test narrative', soap_cat = 'a', episode_id=1, encounter_id=2)
 		#print new_narrative
 
 	#-----------------------------------------
@@ -470,6 +510,3 @@ if __name__ == '__main__':
 
 	#test_search_text_across_emrs()
 	test_narrative()
-
-#============================================================
-
