@@ -14,12 +14,13 @@ import sys
 import time
 import os.path
 import logging
-import codecs
 import re as regex
 import shutil
 import random
 import platform
 import subprocess
+import io
+import codecs
 import socket										# needed for OOo on Windows
 #, libxml2, libxslt
 import shlex
@@ -42,6 +43,7 @@ from Gnumed.pycommon import gmCfg
 from Gnumed.pycommon import gmCfg2
 from Gnumed.pycommon import gmBusinessDBObject
 from Gnumed.pycommon import gmPG2
+from Gnumed.pycommon import gmDateTime
 
 from Gnumed.business import gmPerson
 from Gnumed.business import gmStaff
@@ -654,7 +656,8 @@ class cOOoLetter(object):
 			signal = u'import_document_from_file',
 			filename = filename,
 			document_type = self.instance_type,
-			unlock_patient = True
+			unlock_patient = True,
+			pk_org_unit = gmPraxis.gmCurrentPraxisBranch()['pk_org_unit']
 		)
 		self.ooo_doc = None
 	#--------------------------------------------------------
@@ -789,14 +792,16 @@ class cAbiWordForm(cFormEngine):
 			ext = r'.abw'
 		self.instance_filename = r'%s-instance%s' % (path, ext)
 
-		template_file = codecs.open(self.template_filename, 'rU', 'utf8')
-		instance_file = codecs.open(self.instance_filename, 'wb', 'utf8')
+		template_file = io.open(self.template_filename, mode = 'rt', encoding = 'utf8')
+		instance_file = io.open(self.instance_filename, mode = 'wt', encoding = 'utf8')
 
 		if self.template is not None:
 			# inject placeholder values
 			data_source.set_placeholder(u'form_name_long', self.template['name_long'])
 			data_source.set_placeholder(u'form_name_short', self.template['name_short'])
 			data_source.set_placeholder(u'form_version', self.template['external_version'])
+			data_source.set_placeholder(u'form_version_internal', self.template['gnumed_revision'])
+			data_source.set_placeholder(u'form_last_modified', gmDateTime.pydt_strftime(self.template['last_modified'], '%Y-%b-%d %H:%M'))
 
 		data_source.escape_style = u'xml'
 		data_source.escape_function = None			# gmTools.xml_escape_text() ?
@@ -832,6 +837,8 @@ class cAbiWordForm(cFormEngine):
 			data_source.unset_placeholder(u'form_name_long')
 			data_source.unset_placeholder(u'form_name_short')
 			data_source.unset_placeholder(u'form_version')
+			data_source.unset_placeholder(u'form_version_internal')
+			data_source.unset_placeholder(u'form_last_modified')
 
 		return
 	#--------------------------------------------------------
@@ -880,66 +887,124 @@ class cTextForm(cFormEngine):
 	"""A forms engine outputting data as text for further processing."""
 
 	def __init__(self, template_file=None):
+
 		super(self.__class__, self).__init__(template_file = template_file)
 
-		# generate real template file from .ini file
-		cfg_file = codecs.open(filename = self.template_filename, mode = 'rU', encoding = u'utf8')
+		# create sandbox to play in (and don't assume much
+		# of anything about the template_file except that it
+		# is at our disposal for reading)
+		self.__sandbox_dir = gmTools.mk_sandbox_dir()
+		_log.debug('sandbox directory: [%s]', self.__sandbox_dir)
+
+		# parse template file which is an INI style config
+		# file containing the actual template plus metadata
+		self.form_definition_filename = self.template_filename
+		_log.debug('form definition file: [%s]', self.form_definition_filename)
+		cfg_file = io.open(self.form_definition_filename, mode = 'rt', encoding = u'utf8')
 		self.form_definition = gmCfg2.parse_INI_stream(stream = cfg_file)
 		cfg_file.close()
-		self.form_definition['form::template']
+
+		# extract actual template into a file
+		template_text = self.form_definition['form::template']
+		if isinstance(template_text, type([])):
+			template_text = u'\n'.join(self.form_definition['form::template'])
+		self.template_filename = gmTools.get_unique_filename (
+			prefix = 'gm-',
+			suffix = '.txt',
+			tmp_dir = self.__sandbox_dir
+		)
+		_log.debug('template file: [%s]', self.template_filename)
+		f = io.open(self.template_filename, mode = 'wt', encoding = 'utf8')
+		f.write(template_text)
+		f.close()
+
 	#--------------------------------------------------------
 	def substitute_placeholders(self, data_source=None):
-		self.instance_filename = gmTools.get_unique_filename (
-			prefix = 'gm-T-instance-',
-			suffix = '.txt'
-		)
-		instance_file = codecs.open(self.instance_filename, 'wb', 'utf8')
 
 		if self.template is not None:
 			# inject placeholder values
 			data_source.set_placeholder(u'form_name_long', self.template['name_long'])
 			data_source.set_placeholder(u'form_name_short', self.template['name_short'])
 			data_source.set_placeholder(u'form_version', self.template['external_version'])
+			data_source.set_placeholder(u'form_version_internal', self.template['gnumed_revision'])
+			data_source.set_placeholder(u'form_last_modified', gmDateTime.pydt_strftime(self.template['last_modified'], '%Y-%b-%d %H:%M'))
 
-		if isinstance(self.form_definition['form::template'], type([])):
-			template_text = self.form_definition['form::template']
-		else:
-			template_text = self.form_definition['form::template'].split('\n')
+		base = os.path.join(self.__sandbox_dir, gmTools.fname_stem(self.template_filename))
+		filenames = [
+			self.template_filename,
+			r'%s-result-pass-1.txt' % base,
+			r'%s-result-pass-2.txt' % base,
+			r'%s-result-pass-3.txt' % base
+		]
+		regexen = [
+			'dummy',
+			data_source.first_pass_placeholder_regex,
+			data_source.second_pass_placeholder_regex,
+			data_source.third_pass_placeholder_regex
+		]
 
-		no_errors = True
-		for line in template_text:
+		current_pass = 1
+		while current_pass < 4:
+			_log.debug('placeholder substitution pass #%s', current_pass)
+			found_placeholders = self.__substitute_placeholders (
+				input_filename = filenames[current_pass-1],
+				output_filename = filenames[current_pass],
+				data_source = data_source,
+				placeholder_regex = regexen[current_pass]
+			)
+			current_pass += 1
+
+		# remove temporary placeholders
+		data_source.unset_placeholder(u'form_name_long')
+		data_source.unset_placeholder(u'form_name_short')
+		data_source.unset_placeholder(u'form_version')
+		data_source.unset_placeholder(u'form_version_internal')
+		data_source.unset_placeholder(u'form_last_modified')
+
+		self.instance_filename = self.re_editable_filenames[0]
+
+		return True
+
+	#--------------------------------------------------------
+	def __substitute_placeholders(self, data_source=None, input_filename=None, output_filename=None, placeholder_regex=None):
+
+		_log.debug('[%s] -> [%s]', input_filename, output_filename)
+		_log.debug('searching for placeholders with pattern: %s', placeholder_regex)
+
+		template_file = io.open(input_filename, mode = 'rt', encoding = 'utf8')
+		instance_file = io.open(output_filename, mode = 'wt', encoding = 'utf8')
+
+		for line in template_file:
+			# empty lines
 			if line.strip() in [u'', u'\r', u'\n', u'\r\n']:
-				instance_file.write('%s\n' % line)
+				instance_file.write(line)
 				continue
 
 			# 1) find placeholders in this line
-			placeholders_in_line = regex.findall(data_source.placeholder_regex, line, regex.IGNORECASE)
-			# 2) and replace them
+			placeholders_in_line = regex.findall(placeholder_regex, line, regex.IGNORECASE)
+			if len(placeholders_in_line) == 0:
+				instance_file.write(line)
+				continue
+
+			# 2) replace them
+			_log.debug('%s placeholders found in this line', len(placeholders_in_line))
 			for placeholder in placeholders_in_line:
 				try:
 					val = data_source[placeholder]
 				except:
 					val = _('error with placeholder [%s]') % placeholder
 					_log.exception(val)
-					no_errors = False
-
 				if val is None:
 					val = _('error with placeholder [%s]') % placeholder
 
 				line = line.replace(placeholder, val)
 
-			instance_file.write(u'%s\n' % line)
+			instance_file.write(line)
 
 		instance_file.close()
-		self.re_editable_filenames = [self.instance_filename]
+		self.re_editable_filenames = [output_filename]
+		template_file.close()
 
-		if self.template is not None:
-			# remove temporary placeholders
-			data_source.unset_placeholder(u'form_name_long')
-			data_source.unset_placeholder(u'form_name_short')
-			data_source.unset_placeholder(u'form_version')
-
-		return no_errors
 	#--------------------------------------------------------
 	def edit(self):
 
@@ -961,15 +1026,19 @@ class cTextForm(cFormEngine):
 		self.re_editable_filenames = [self.instance_filename]
 
 		return result
+
 	#--------------------------------------------------------
 	def generate_output(self, format=None):
 		try:
-			post_processor = self.form_definition['form::post processor'] % self.instance_filename
+			post_processor = self.form_definition['form::post processor'] % {
+				'input_name': self.instance_filename,
+				'output_name': self.instance_filename + u'.output'
+			}
 		except KeyError:
 			_log.debug('no explicit post processor defined for text template')
 			return True
 
-		self.final_output_filenames = [self.instance_filename]
+		self.final_output_filenames = [self.instance_filename + u'.output']
 
 		return gmShellAPI.run_command_in_shell(command = post_processor, blocking = True)
 #------------------------------------------------------------
@@ -985,19 +1054,16 @@ class cLaTeXForm(cFormEngine):
 
 		# create sandbox for LaTeX to play in (and don't assume
 		# much of anything about the template_file except that it
-		# is at our disposal)
-		sandbox_dir = gmTools.get_unique_filename (
-			prefix = gmTools.fname_stem(template_file) + '_',
-			suffix = '.dir'
-		)
+		# is at our disposal for reading)
+		sandbox_dir = gmTools.mk_sandbox_dir(prefix = gmTools.fname_stem(template_file) + '_')
 		_log.debug('LaTeX sandbox directory: [%s]', sandbox_dir)
-		gmTools.mkdir(sandbox_dir)
 		shutil.copy(template_file, sandbox_dir)
 		template_file = os.path.join(sandbox_dir, os.path.split(template_file)[1])
 
 		super(self.__class__, self).__init__(template_file = template_file)
 
 		self.__sandbox_dir = sandbox_dir
+
 	#--------------------------------------------------------
 	def substitute_placeholders(self, data_source=None):
 
@@ -1006,6 +1072,8 @@ class cLaTeXForm(cFormEngine):
 			data_source.set_placeholder(u'form_name_long', self.template['name_long'])
 			data_source.set_placeholder(u'form_name_short', self.template['name_short'])
 			data_source.set_placeholder(u'form_version', self.template['external_version'])
+			data_source.set_placeholder(u'form_version_internal', self.template['gnumed_revision'])
+			data_source.set_placeholder(u'form_last_modified', gmDateTime.pydt_strftime(self.template['last_modified'], '%Y-%b-%d %H:%M'))
 
 		data_source.escape_function = gmTools.tex_escape_string
 		data_source.escape_style = u'latex'
@@ -1016,78 +1084,83 @@ class cLaTeXForm(cFormEngine):
 
 		filenames = [
 			self.template_filename,
-			r'%s-result_run1%s' % (path, ext),
-			r'%s-result_run2%s' % (path, ext),
-			r'%s-result_run3%s' % (path, ext)
+			r'%s-result-pass-1%s' % (path, ext),
+			r'%s-result-pass-2%s' % (path, ext),
+			r'%s-result-pass-3%s' % (path, ext)
+		]
+		regexen = [
+			'dummy',
+			data_source.first_pass_placeholder_regex,
+			data_source.second_pass_placeholder_regex,
+			data_source.third_pass_placeholder_regex
 		]
 
-		found_placeholders = True
-		current_run = 1
-		while found_placeholders and (current_run < 4):
-			_log.debug('placeholder substitution run #%s', current_run)
+		current_pass = 1
+		while current_pass < 4:
+			_log.debug('placeholder substitution pass #%s', current_pass)
 			found_placeholders = self.__substitute_placeholders (
-				input_filename = filenames[current_run-1],
-				output_filename = filenames[current_run],
-				data_source = data_source
+				input_filename = filenames[current_pass-1],
+				output_filename = filenames[current_pass],
+				data_source = data_source,
+				placeholder_regex = regexen[current_pass]
 			)
-			current_run += 1
+			current_pass += 1
 
-		if self.template is not None:
-			# remove temporary placeholders
-			data_source.unset_placeholder(u'form_name_long')
-			data_source.unset_placeholder(u'form_name_short')
-			data_source.unset_placeholder(u'form_version')
+		# remove temporary placeholders
+		data_source.unset_placeholder(u'form_name_long')
+		data_source.unset_placeholder(u'form_name_short')
+		data_source.unset_placeholder(u'form_version')
+		data_source.unset_placeholder(u'form_version_internal')
+		data_source.unset_placeholder(u'form_last_modified')
 
 		self.instance_filename = self.re_editable_filenames[0]
 
 		return
 	#--------------------------------------------------------
-	def __substitute_placeholders(self, data_source=None, input_filename=None, output_filename=None):
+	def __substitute_placeholders(self, data_source=None, input_filename=None, output_filename=None, placeholder_regex=None):
 
 		_log.debug('[%s] -> [%s]', input_filename, output_filename)
+		_log.debug('searching for placeholders with pattern: %s', placeholder_regex)
 
-		found_placeholders = False
-
-		template_file = codecs.open(input_filename, 'rU', 'utf8')
-		instance_file = codecs.open(output_filename, 'wb', 'utf8')
+		template_file = io.open(input_filename, mode = 'rt', encoding = 'utf8')
+		instance_file = io.open(output_filename, mode = 'wt', encoding = 'utf8')
 
 		for line in template_file:
-
-			if line.strip() in [u'', u'\r', u'\n', u'\r\n']:		# empty lines
+			# empty lines
+			if line.strip() in [u'', u'\r', u'\n', u'\r\n']:
 				instance_file.write(line)
 				continue
-			if line.lstrip().startswith('%'):		# TeX comment
+			# TeX-comment-only lines
+			if line.lstrip().startswith('%'):
 				instance_file.write(line)
 				continue
 
-			for placeholder_regex in [data_source.first_order_placeholder_regex, data_source.second_order_placeholder_regex, data_source.third_order_placeholder_regex]:
-				# 1) find placeholders in this line
-				placeholders_in_line = regex.findall(placeholder_regex, line, regex.IGNORECASE)
-				if len(placeholders_in_line) == 0:
-					continue
-				_log.debug('%s placeholders found with pattern: %s', len(placeholders_in_line), placeholder_regex)
-				found_placeholders = True
-				# 2) replace them
-				for placeholder in placeholders_in_line:
-					try:
-						val = data_source[placeholder]
-					except:
-						_log.exception('error with placeholder [%s]', placeholder)
-						val = gmTools.tex_escape_string(_('error with placeholder [%s]') % placeholder)
-
-					if val is None:
-						_log.debug('error with placeholder [%s]', placeholder)
-						val = _('error with placeholder [%s]') % gmTools.tex_escape_string(placeholder)
-
-					line = line.replace(placeholder, val)
+			# 1) find placeholders in this line
+			placeholders_in_line = regex.findall(placeholder_regex, line, regex.IGNORECASE)
+			if len(placeholders_in_line) == 0:
+				instance_file.write(line)
+				continue
 
+			# 2) replace them
+			_log.debug('%s placeholders found in this line', len(placeholders_in_line))
+			for placeholder in placeholders_in_line:
+				try:
+					val = data_source[placeholder]
+				except:
+					_log.exception('error with placeholder [%s]', placeholder)
+					val = gmTools.tex_escape_string(_('error with placeholder [%s]') % placeholder)
+				if val is None:
+					_log.debug('error with placeholder [%s]', placeholder)
+					val = gmTools.tex_escape_string(_('error with placeholder [%s]') % placeholder)
+				line = line.replace(placeholder, val)
 			instance_file.write(line)
 
 		instance_file.close()
 		self.re_editable_filenames = [output_filename]
 		template_file.close()
 
-		return found_placeholders
+		return
+
 	#--------------------------------------------------------
 	def edit(self):
 
@@ -1178,12 +1251,8 @@ class cXeTeXForm(cFormEngine):
 		# create sandbox for LaTeX to play in (and don't assume
 		# much of anything about the template_file except that it
 		# is at our disposal)
-		sandbox_dir = gmTools.get_unique_filename (
-			prefix = gmTools.fname_stem(template_file) + '_',
-			suffix = '.dir'
-		)
+		sandbox_dir = gmTools.mk_sandbox_dir(prefix = gmTools.fname_stem(template_file) + '_')
 		_log.debug('Xe(La)TeX sandbox directory: [%s]', sandbox_dir)
-		gmTools.mkdir(sandbox_dir)
 		shutil.copy(template_file, sandbox_dir)
 		template_file = os.path.join(sandbox_dir, os.path.split(template_file)[1])
 
@@ -1198,6 +1267,8 @@ class cXeTeXForm(cFormEngine):
 			data_source.set_placeholder(u'form_name_long', self.template['name_long'])
 			data_source.set_placeholder(u'form_name_short', self.template['name_short'])
 			data_source.set_placeholder(u'form_version', self.template['external_version'])
+			data_source.set_placeholder(u'form_version_internal', self.template['gnumed_revision'])
+			data_source.set_placeholder(u'form_last_modified', gmDateTime.pydt_strftime(self.template['last_modified'], '%Y-%b-%d %H:%M'))
 
 		data_source.escape_function = gmTools.xetex_escape_string
 		data_source.escape_style = u'xetex'
@@ -1229,6 +1300,8 @@ class cXeTeXForm(cFormEngine):
 			data_source.unset_placeholder(u'form_name_long')
 			data_source.unset_placeholder(u'form_name_short')
 			data_source.unset_placeholder(u'form_version')
+			data_source.unset_placeholder(u'form_version_internal')
+			data_source.unset_placeholder(u'form_last_modified')
 
 		self.instance_filename = self.re_editable_filenames[0]
 
@@ -1239,8 +1312,8 @@ class cXeTeXForm(cFormEngine):
 
 		found_placeholders = False
 
-		template_file = codecs.open(input_filename, 'rU', 'utf8')
-		instance_file = codecs.open(output_filename, 'wb', 'utf8')
+		template_file = io.open(input_filename, mode = 'rt', encoding = 'utf8')
+		instance_file = io.open(output_filename, mode = 'wt', encoding = 'utf8')
 
 		for line in template_file:
 
@@ -1251,7 +1324,7 @@ class cXeTeXForm(cFormEngine):
 				instance_file.write(line)
 				continue
 
-			for placeholder_regex in [data_source.first_order_placeholder_regex, data_source.second_order_placeholder_regex, data_source.third_order_placeholder_regex]:
+			for placeholder_regex in [data_source.first_pass_placeholder_regex, data_source.second_pass_placeholder_regex, data_source.third_pass_placeholder_regex]:
 				# 1) find placeholders in this line
 				placeholders_in_line = regex.findall(placeholder_regex, line, regex.IGNORECASE)
 				if len(placeholders_in_line) == 0:
@@ -1385,9 +1458,9 @@ class cGnuplotForm(cFormEngine):
 		Expects .data_filename to be set.
 		"""
 		self.conf_filename = gmTools.get_unique_filename(prefix = 'gm2gpl-', suffix = '.conf')
-		conf_file = codecs.open(self.conf_filename, 'wb', 'utf8')
-		conf_file.write('# setting the gnuplot data file\n')
-		conf_file.write("gm2gpl_datafile = '%s'\n" % self.data_filename)
+		conf_file = io.open(self.conf_filename, mode = 'wt', encoding = 'utf8')
+		conf_file.write(u'# setting the gnuplot data file\n')
+		conf_file.write(u"gm2gpl_datafile = '%s'\n" % self.data_filename)
 		conf_file.close()
 
 		# FIXME: cater for configurable path
@@ -1396,7 +1469,12 @@ class cGnuplotForm(cFormEngine):
 		else:
 			exec_name = 'gnuplot'
 
-		args = [exec_name, '-p', self.conf_filename, self.template_filename]
+		args = [
+			exec_name,
+			'-p',					# let plot window persist after main gnuplot process exits
+			self.conf_filename,		# contains the gm2gpl_datafile setting which, in turn, contains the actual data
+			self.template_filename	# contains the plotting instructions (IOW a user provided gnuplot script)
+		]
 		_log.debug('plotting args: %s' % str(args))
 
 		try:
@@ -1478,8 +1556,8 @@ class cPDFForm(cFormEngine):
 
 		# parse dumped FDF file for "/V (...)" records
 		# and replace placeholders therein
-		fdf_dumped_file = open(self.fdf_dumped_filename, 'rbU')
-		fdf_replaced_file = codecs.open(self.fdf_replaced_filename, 'wb')
+		fdf_dumped_file = io.open(self.fdf_dumped_filename, mode = 'rt', encoding = u'utf8')
+		fdf_replaced_file = io.open(self.fdf_replaced_filename, mode = 'wt', encoding = u'utf8')
 
 		string_value_regex = r'\s*/V\s*\(.+\)\s*$'
 		for line in fdf_dumped_file:
@@ -1533,10 +1611,10 @@ class cPDFForm(cFormEngine):
 
 			replaced_line = '\x00\\n'.join(replaced_lines)
 
-			fdf_replaced_file.write('/V (')
+			fdf_replaced_file.write(u'/V (')
 			fdf_replaced_file.write(codecs.BOM_UTF16_BE)
 			fdf_replaced_file.write(replaced_line)
-			fdf_replaced_file.write(')\n')
+			fdf_replaced_file.write(u')\n')
 
 		fdf_replaced_file.close()
 		fdf_dumped_file.close()
@@ -1768,7 +1846,7 @@ class cXSLTFormEngine(cFormEngine):
 		fname = gmTools.get_unique_filename(prefix = u'gm_XSLT_form-', suffix = u'.html')
 		#html_file = os.open(fname, 'wb')
 		#html_file.write(self._FormData.encode('UTF-8'))
-		html_file = codecs.open(fname, 'wb', 'utf8', 'strict')		# or 'replace' ?
+		html_file = io.open(fname, mode = 'wt', encoding = 'utf8', errors = 'strict')		# or 'replace' ?
 		html_file.write(self._FormData)
 		html_file.close()
 
@@ -1913,7 +1991,7 @@ $DISEASELIST
 
 
 def test_au():
-	f = open('../../test-area/ian/terry-form.tex')
+	f = io.open('../../test-area/ian/terry-form.tex')
 	params = {
 	'RECIPIENT': "Dr. R. Terry\n1 Main St\nNewcastle",
 	'DOCTORSNAME': 'Ian Haywood',
@@ -1961,7 +2039,7 @@ def test_au2 ():
 	form.cleanup ()
 #------------------------------------------------------------
 def test_de():
-		template = open('../../test-area/ian/Formularkopf-DE.tex')
+		template = io.open('../../test-area/ian/Formularkopf-DE.tex')
 		form = LaTeXForm(template=template.read())
 		params = {
 				'PATIENT LASTNAME': 'Kirk',
@@ -1986,7 +2064,6 @@ if __name__ == '__main__':
 	if sys.argv[1] != 'test':
 		sys.exit()
 
-	from Gnumed.pycommon import gmDateTime
 	gmDateTime.init()
 
 	#--------------------------------------------------------
@@ -2131,6 +2208,13 @@ if __name__ == '__main__':
 		print "final file is:", final_name
 	#--------------------------------------------------------
 	def test_text_form():
+
+		from Gnumed.business import gmPraxis
+
+		branches = gmPraxis.get_praxis_branches()
+		praxis = gmPraxis.gmCurrentPraxisBranch(branches[0])
+		print praxis
+
 		pat = gmPersonSearch.ask_for_patient()
 		if pat is None:
 			return
@@ -2145,6 +2229,7 @@ if __name__ == '__main__':
 		ph = gmMacro.gmPlaceholderHandler()
 		ph.debug = True
 		print "placeholder substitution worked:", form.substitute_placeholders(data_source = ph)
+		print form.re_editable_filenames
 		form.edit()
 		form.generate_output()
 	#--------------------------------------------------------
@@ -2164,9 +2249,9 @@ if __name__ == '__main__':
 
 	#test_cFormTemplate()
 	#set_template_from_file()
-	test_latex_form()
+	#test_latex_form()
 	#test_pdf_form()
 	#test_abiword_form()
-	#test_text_form()
+	test_text_form()
 
 #============================================================
