@@ -81,6 +81,7 @@ class cOrg(gmBusinessDBObject.cBusinessDBObject):
 		return get_org_units(order_by = u'unit', org = self._payload[self._idx['pk_org']])
 
 	units = property(_get_units, lambda x:x)
+
 #------------------------------------------------------------
 def org_exists(organization=None, category=None, link_obj=None):
 	args = {'desc': organization, 'cat': category}
@@ -200,6 +201,9 @@ class cOrgUnit(gmBusinessDBObject.cBusinessDBObject):
 					'idx': idx
 				}) for r in rows
 			]
+
+	comm_channels = property(get_comm_channels, lambda x:x)
+
 	#--------------------------------------------------------
 	def link_comm_channel(self, comm_medium=None, url=None, is_confidential=False, pk_channel_type=None):
 		"""Link a communication medium with this org unit.
@@ -384,6 +388,7 @@ class cOrgUnit(gmBusinessDBObject.cBusinessDBObject):
 					gmTools.bool2subst(comm['is_confidential'], _(' (confidential)'), u'', u'')
 				))
 		return lines
+
 	#--------------------------------------------------------
 	# properties
 	#--------------------------------------------------------
@@ -397,6 +402,7 @@ class cOrgUnit(gmBusinessDBObject.cBusinessDBObject):
 		self.save()
 
 	address = property(_get_address, _set_address)
+
 	#--------------------------------------------------------
 	def _get_org(self):
 		return cOrg(aPK_obj = self._payload[self._idx['pk_org']])
@@ -405,6 +411,7 @@ class cOrgUnit(gmBusinessDBObject.cBusinessDBObject):
 	org = property(_get_org, lambda x:x)
 
 	comm_channels = property(get_comm_channels, lambda x:x)
+
 #------------------------------------------------------------
 def create_org_unit(pk_organization=None, unit=None, link_obj=None):
 	_log.debug(u'creating org unit [%s:%s]', unit, pk_organization)
@@ -519,7 +526,7 @@ def get_comm_channels_data_for_org_ids( idList):
 		return None 
 	m = {}
 	for (id_org, id_type, url) in result:
-		if not m.has_key(id_org):
+		if id_org not in m:
 			m[id_org] = []
 		m[id_org].append( (id_type, url) )
 
@@ -531,7 +538,7 @@ def get_address_data_for_org_ids( idList):
 	"""
 
 	ids = ", ".join( [ str(x) for x in idList]) 
-	cmd = """select l.id_org, number, street, city, postcode, state, country 
+	cmd = """select l.id_org, number, street, city, postcode, region, country 
 			from dem.v_basic_address v , dem.lnk_org2address l 
 				where v.addr_id = l.id_address and
 				l.id_org in ( select id from dem.org where id in (%s) ) """ % ids 
@@ -736,7 +743,7 @@ if __name__ == '__main__':
 
 		print """testing single level orgs"""
 		f = [ "name", "office", "subtype",  "memo", "category", "phone", "fax", "email","mobile"]
-		a = ["number", "street", "urb", "postcode", "state", "country"]
+		a = ["number", "street", "urb", "postcode", 'region', "country"]
 		h = cOrgImpl1()
 
 		h.set(*f1)
@@ -849,7 +856,7 @@ if __name__ == '__main__':
 			("""drop table del_org""", [])
 
 			]
-		result =  gmPG.run_commit("personalia", cmds) <> None
+		result = (gmPG.run_commit("personalia", cmds) is not None)
 
 		return result
 
@@ -929,7 +936,7 @@ if __name__ == '__main__':
 	def create_temp_categories( categories = ['hospital']):
 		print "NEED TO CREATE TEMPORARY ORG_CATEGORY.\n\n ** PLEASE ENTER administrator login  : e.g  user 'gm-dbo' and  his password"
 		#get a admin login
-		for i in xrange(0, 4):
+		for i in range(0, 4):
 			result ,tmplogin = login_admin_user()
 			if result:
 				break
@@ -980,7 +987,7 @@ if __name__ == '__main__':
 		Please enter login for administrator:
 		"""
 		if adminlogin is None:
-			for i in xrange(0, 4):
+			for i in range(0, 4):
 				result, adminlogin = login_admin_user()
 				if  result:
 					break
@@ -1012,7 +1019,7 @@ if __name__ == '__main__':
 
 		conn = None
 		p.ReleaseConnection(service)
-		if failed_remove <> []:
+		if failed_remove != []:
 			print "FAILED TO REMOVE ", failed_remove
 		return failed_remove
 
@@ -1214,7 +1221,7 @@ if __name__ == '__main__':
 		if result == categories:
 			print "Unable to create temporary org_category. Test aborted"
 			sys.exit(-1)
-		if result <> []:
+		if result != []:
 			print "UNABLE TO CREATE THESE CATEGORIES"
 			if not raw_input("Continue ?") in ['y', 'Y'] :
 				sys.exit(-1)
@@ -1245,7 +1252,7 @@ if __name__ == '__main__':
 
 			# cleanup after the test case
 		for (result , org) in results:
-			if not result and org.getId() <> None:
+			if not result and org.getId() is not None:
 				print "trying cleanup"
 				if  org.shallow_del(): print " 	may have succeeded"
 				else:
