@@ -6,6 +6,7 @@ __author__ = "K.Hilbert <Karsten.Hilbert@gmx.net>"
 
 import sys
 import logging
+import io
 
 
 if __name__ == '__main__':
@@ -103,6 +104,7 @@ class cPraxisBranch(gmBusinessDBObject.cBusinessDBObject):
 		return gmOrganization.cOrgUnit(aPK_obj = self._payload[self._idx['pk_org_unit']])
 
 	org_unit = property(_get_org_unit, lambda x:x)
+
 	#--------------------------------------------------------
 	def _get_org(self):
 		return gmOrganization.cOrg(aPK_obj = self._payload[self._idx['pk_org']])
@@ -120,6 +122,37 @@ class cPraxisBranch(gmBusinessDBObject.cBusinessDBObject):
 #		self.save()
 #	address = property(_get_address, _set_address)
 
+	#--------------------------------------------------------
+	def _get_vcf(self):
+		vcf_fields = [
+			u'BEGIN:VCARD',
+			u'VERSION:4.0',
+			u'KIND:org',
+			_(u'FN:%(l10n_unit_category)s %(branch)s of %(l10n_organization_category)s %(praxis)s') % self,
+			u'N:%(praxis)s;%(branch)s' % self
+		]
+		adr = self.address
+		if adr is not None:
+			vcf_fields.append(u'ADR:;%(subunit)s;%(street)s %(number)s;%(urb)s;%(l10n_region)s;%(postcode)s;%(l10n_country)s' % adr)
+		comms = self.get_comm_channels(comm_medium = u'workphone')
+		if len(comms) > 0:
+			vcf_fields.append(u'TEL;VALUE=uri;TYPE=work:tel:%(url)s' % comms[0])
+		comms = self.get_comm_channels(comm_medium = u'email')
+		if len(comms) > 0:
+			vcf_fields.append(u'EMAIL:%(url)s' % comms[0])
+		vcf_fields.append(u'END:VCARD')
+		vcf_fname = gmTools.get_unique_filename (
+			prefix = 'gm_praxis2vcf-',
+			suffix = '.vcf'
+		)
+		vcf_file = io.open(vcf_fname, mode = 'wt', encoding = 'utf8')
+		vcf_file.write(u'\n'.join(vcf_fields))
+		vcf_file.write(u'\n')
+		vcf_file.close()
+		return vcf_fname
+
+	vcf = property(_get_vcf, lambda x:x)
+
 #------------------------------------------------------------
 def lock_praxis_branch(pk_praxis_branch=None, exclusive=False):
 	return gmPG2.lock_row(table = u'dem.praxis_branch', pk = pk_praxis_branch, exclusive = exclusive)
@@ -273,15 +306,18 @@ class gmCurrentPraxisBranch(gmBorg.cBorg):
 		if attribute in ['branch', 'waiting_list_patients', 'help_desk', 'db_logon_banner', 'active_workplace', 'workplaces', 'user_email']:
 			return getattr(self, attribute)
 		return getattr(self.branch, attribute)
+
 	#--------------------------------------------------------
 	# __get/setitem__ handling
 	#--------------------------------------------------------
 	def __getitem__(self, attribute = None):
 		"""Return any attribute if known how to retrieve it by proxy."""
 		return self.branch[attribute]
+
 	#--------------------------------------------------------
 	def __setitem__(self, attribute, value):
 		self.branch[attribute] = value
+
 	#--------------------------------------------------------
 	# waiting list handling
 	#--------------------------------------------------------
@@ -338,6 +374,7 @@ where
 		return rows
 
 	waiting_list_patients = property (_get_waiting_list_patients, lambda x:x)
+
 	#--------------------------------------------------------
 	def _set_helpdesk(self, helpdesk):
 		return
@@ -365,6 +402,7 @@ where
 		return self.__helpdesk
 
 	helpdesk = property(_get_helpdesk, _set_helpdesk)
+
 	#--------------------------------------------------------
 	def _get_db_logon_banner(self):
 		rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': u'select _(message) from cfg.db_logon_banner'}])
@@ -384,6 +422,7 @@ where
 		rows, idx = gmPG2.run_rw_queries(queries = queries, end_tx = True)
 
 	db_logon_banner = property(_get_db_logon_banner, _set_db_logon_banner)
+
 	#--------------------------------------------------------
 	def _set_workplace(self, workplace):
 		# maybe later allow switching workplaces on the fly
@@ -415,6 +454,7 @@ where
 		return self.__active_workplace
 
 	active_workplace = property(_get_workplace, _set_workplace)
+
 	#--------------------------------------------------------
 	def _set_workplaces(self, val):
 		pass
@@ -424,6 +464,7 @@ where
 		return [ r[0] for r in rows ]
 
 	workplaces = property(_get_workplaces, _set_workplaces)
+
 	#--------------------------------------------------------
 	def _get_user_email(self):
 		# FIXME: get this from the current users staff record in the database
