@@ -1,5 +1,7 @@
 #!/usr/bin/env python
 
+from __future__ import print_function
+
 __doc__ = """GNUmed client launcher.
 
 This is the launcher for the GNUmed GUI client. It takes
@@ -62,32 +64,32 @@ import signal
 import os.path
 import shutil
 import stat
-import codecs
+import io
 
 
 # do not run as module
 if __name__ != "__main__":
-	print "GNUmed startup: This is not intended to be imported as a module !"
-	print "-----------------------------------------------------------------"
-	print __doc__
+	print("GNUmed startup: This is not intended to be imported as a module !")
+	print("-----------------------------------------------------------------")
+	print(__doc__)
 	sys.exit(1)
 
 
 # do not run as root
 if os.name in ['posix'] and os.geteuid() == 0:
-	print """
+	print("""
 GNUmed startup: GNUmed should not be run as root.
 -------------------------------------------------
 
 Running GNUmed as <root> can potentially put all
 your medical data at risk. It is strongly advised
 against. Please run GNUmed as a non-root user.
-"""
+""")
 	sys.exit(1)
 
 #----------------------------------------------------------
-current_client_version = u'1.5.10'
-current_client_branch = u'1.5'
+current_client_version = u'1.6.5'
+current_client_branch = u'1.6'
 
 _log = None
 _pre_log_buffer = []
@@ -192,7 +194,7 @@ def setup_python_path():
 	local_python_import_dir = os.path.dirname (
 		os.path.abspath(os.path.join(sys.argv[0], '..'))
 	)
-	print "Running from local source tree (%s) ..." % local_python_import_dir
+	print("Running from local source tree (%s) ..." % local_python_import_dir)
 	_pre_log_buffer.append("running from local source tree: %s" % local_python_import_dir)
 
 	# does the path exist at all, physically ?
@@ -202,9 +204,9 @@ def setup_python_path():
 		_pre_log_buffer.append('local module import dir symlink exists: %s' % link_name)
 	else:
 		real_dir = os.path.join(local_python_import_dir, 'client')
-		print "Creating local module import symlink ..."
-		print ' real dir:', real_dir
-		print '     link:', link_name
+		print('Creating local module import symlink ...')
+		print(' real dir:', real_dir)
+		print('     link:', link_name)
 		try:
 			os.symlink(real_dir, link_name)
 		except AttributeError:
@@ -213,7 +215,7 @@ def setup_python_path():
 			_pre_log_buffer.append('ctypes.windll.kernel32.CreateSymbolicLinkW() exit code: %s', result)
 		_pre_log_buffer.append('created local module import dir symlink: link [%s] => dir [%s]' % (link_name, real_dir))
 
-	print "Adjusting PYTHONPATH ..."
+	print("Adjusting PYTHONPATH ...")
 	sys.path.insert(0, local_python_import_dir)
 	_pre_log_buffer.append('sys.path with local module import base dir prepended: %s' % sys.path)
 
@@ -269,7 +271,7 @@ def setup_local_repo_path():
 			_log.error('insecure repo: permissions not 0600')
 			return
 
-	print "Activating local wxGlade widgets repository (%s) ..." % local_wxGladeWidgets_path
+	print("Activating local wxGlade widgets repository (%s) ..." % local_wxGladeWidgets_path)
 	sys.path.insert(0, local_repo_path)
 	_log.debug('sys.path with repo:')
 	_log.debug(sys.path)
@@ -279,7 +281,7 @@ def setup_fault_handler(target=None):
 	try:
 		import faulthandler
 	except ImportError:
-		print "Faulthandler not available ..."
+		print("Faulthandler not available ...")
 		_pre_log_buffer.append('<faulthandler> not available')
 		return
 	if target is None:
@@ -296,7 +298,7 @@ def setup_logging():
 	except ImportError:
 		sys.exit(import_error_sermon % '\n '.join(sys.path))
 
-	print "Log file: %s" % _gmLog2._logfile.name
+	print("Log file:", _gmLog2._logfile.name)
 	setup_fault_handler(target = _gmLog2._logfile)
 
 	global gmLog2
@@ -326,7 +328,8 @@ def log_startup_info():
 	for key, val in os.environ.items():
 		_log.info(u' %s: %s' % (
 			(u'${%s}' % key).rjust(30),
-			unicode(val, encoding = sys.getfilesystemencoding(), errors = 'replace')
+			# this won't work in Python3 because that'll be a byte sequence, not a string and thus will need .ENcode
+			val.decode(encoding = sys.getfilesystemencoding(), errors = 'replace')
 		))
 
 #==========================================================
@@ -400,9 +403,9 @@ def setup_cli():
 def handle_sig_term(signum, frame):
 	_log.critical('SIGTERM (SIG%s) received, shutting down ...' % signum)
 	gmLog2.flush()
-	print 'GNUmed: SIGTERM (SIG%s) received, shutting down ...' % signum
+	print('GNUmed: SIGTERM (SIG%s) received, shutting down ...' % signum)
 	if frame is not None:
-		print '%s::%s@%s' % (frame.f_code.co_filename, frame.f_code.co_name, frame.f_lineno)
+		print('%s::%s@%s' % (frame.f_code.co_filename, frame.f_code.co_name, frame.f_lineno))
 
 	# FIXME: need to do something useful here
 
@@ -438,11 +441,11 @@ def handle_help_request():
 	)
 
 	if help_requested:
-		print _(
+		print(_(
 			'Help requested\n'
 			'--------------'
-		)
-		print __doc__
+		))
+		print(__doc__)
 		sys.exit(0)
 #==========================================================
 def handle_version_request():
@@ -457,11 +460,11 @@ def handle_version_request():
 
 		from Gnumed.pycommon.gmPG2 import map_client_branch2required_db_version, known_schema_hashes
 
-		print 'GNUmed version information'
-		print '--------------------------'
-		print 'client     : %s on branch [%s]' % (current_client_version, current_client_branch)
-		print 'database   : %s' % map_client_branch2required_db_version[current_client_branch]
-		print 'schema hash: %s' % known_schema_hashes[map_client_branch2required_db_version[current_client_branch]]
+		print('GNUmed version information')
+		print('--------------------------')
+		print('client     : %s on branch [%s]' % (current_client_version, current_client_branch))
+		print('database   : %s' % map_client_branch2required_db_version[current_client_branch])
+		print('schema hash: %s' % known_schema_hashes[map_client_branch2required_db_version[current_client_branch]])
 		sys.exit(0)
 
 #==========================================================
@@ -495,13 +498,13 @@ the hidden directory "%s/".""" % (
 	gmTools.mkdir(os.path.expanduser(os.path.join('~', '.gnumed', 'error_logs')))
 	gmTools.mkdir(os.path.expanduser(os.path.join('~', 'gnumed')))
 
-	README = codecs.open(os.path.expanduser(os.path.join('~', 'gnumed', '00_README')), 'wb', 'utf8')
+	README = io.open(os.path.expanduser(os.path.join('~', 'gnumed', '00_README')), mode = 'wt', encoding = 'utf8')
 	README.write(gnumed_DIR_README_TEXT)
 	README.close()
 
 	paths = gmTools.gmPaths(app_name = u'gnumed')
 
-	open(os.path.expanduser(os.path.join('~', '.gnumed', 'gnumed.conf')), 'a+').close()
+	io.open(os.path.expanduser(os.path.join('~', '.gnumed', 'gnumed.conf')), mode = 'a+t').close()
 #==========================================================
 def setup_date_time():
 	gmDateTime.init()
@@ -582,7 +585,7 @@ def setup_ui_type():
 	if ui_type not in _known_ui_types:
 		_log.error('unknown UI type: %s', ui_type)
 		_log.debug('known UI types: %s', str(_known_ui_types))
-		print "GNUmed startup: Unknown UI type (%s). Defaulting to wxPython client." % ui_type
+		print("GNUmed startup: Unknown UI type (%s). Defaulting to wxPython client." % ui_type)
 		ui_type = 'wxp'
 
 	_log.debug('UI type: %s', ui_type)
@@ -633,10 +636,10 @@ def shutdown_logging():
 #			pairs.reverse()
 #			return pairs
 
-#		rcfile = open('./gm-refcount.lst', 'wb')
+#		rcfile = io.open('./gm-refcount.lst', 'wt', encoding = 'utf8')
 #		for refcount, class_ in get_refcounts():
 #			if not class_.__name__.startswith('wx'):
-#				rcfile.write('%10d %s\n' % (refcount, class_.__name__))
+#				rcfile.write(u'%10d %s\n' % (refcount, class_.__name__))
 #		rcfile.close()
 
 	# do not choke on Windows
@@ -690,13 +693,13 @@ if ui_type == u'wxp':
 		profile.run('gmGuiMain.main()', profile_file)
 	else:
 		gmGuiMain.main()
-elif ui_type == u'web':
-	from Gnumed.proxiedpyjamas import gmWebGuiServer
-	gmWebGuiServer.main()
+#elif ui_type == u'web':
+#	from Gnumed.proxiedpyjamas import gmWebGuiServer
+#	gmWebGuiServer.main()
 
-elif ui_type == u'chweb':
-	from Gnumed.CherryPy import gmGuiWeb
-	gmGuiWeb.main()
+#elif ui_type == u'chweb':
+#	from Gnumed.CherryPy import gmGuiWeb
+#	gmGuiWeb.main()
 
 gmHooks.run_hook_script(hook = u'shutdown-post-GUI')
 
