@@ -1,4 +1,7 @@
-"""GNUmed client internationalization/localization.
+
+from __future__ import print_function
+
+__doc__ = """GNUmed client internationalization/localization.
 
 All i18n/l10n issues should be handled through this modules.
 
@@ -53,8 +56,14 @@ __license__ = "GPL v2 or later (details at http://www.gnu.org)"
 
 
 # stdlib
-import sys, os.path, os, re as regex, locale, gettext, logging, codecs
-
+import sys
+import os.path
+import os
+import locale
+import gettext
+import logging
+import codecs
+import re as regex
 
 _log = logging.getLogger('gm.i18n')
 
@@ -102,6 +111,7 @@ def __split_locale_into_levels():
 	system_locale_level['language'] = system_locale.split('_', 1)[0]
 
 	_log.debug('system locale levels: %s', system_locale_level)
+
 #---------------------------------------------------------------------------
 def __log_locale_settings(message=None):
 	_setlocale_categories = {}
@@ -196,6 +206,7 @@ def __log_locale_settings(message=None):
 		_log.exception('this OS does not support nl_langinfo')
 
 	_log.debug('gmI18N.get_encoding(): %s', get_encoding())
+
 #---------------------------------------------------------------------------
 def _translate_protected(term):
 	"""This wraps _().
@@ -211,24 +222,17 @@ def _translate_protected(term):
 		_log.error('translation: %s', translation)
 		return term
 
-	term_substitutes = _substitutes_regex.findall(term)
-	trans_substitutes = _substitutes_regex.findall(translation)
+	substitution_keys_in_original = set(_substitutes_regex.findall(term))
+	substitution_keys_in_translation = set(_substitutes_regex.findall(translation))
 
-	# different number of %(...)s substitutes ?
-	if len(term_substitutes) != len(trans_substitutes):
-		_log.error('count("%(...)s") mismatch, returning untranslated string')
-		_log.error('original   : %s', term)
-		_log.error('translation: %s', translation)
-		return term
-
-	# different %(...)s substitutes ?
-	if set(term_substitutes) != set(trans_substitutes):
-		_log.error('"%(...)s" name mismatch, returning untranslated string')
+	if not substitution_keys_in_translation.issubset(substitution_keys_in_original):
+		_log.error('"%(...)s" keys in translation not a subset of keys in original, returning untranslated string')
 		_log.error('original   : %s', term)
 		_log.error('translation: %s', translation)
 		return term
 
 	return translation
+
 #---------------------------------------------------------------------------
 # external API
 #---------------------------------------------------------------------------
@@ -236,7 +240,6 @@ def activate_locale():
 	"""Get system locale from environment."""
 	global system_locale
 
-	# logging state of affairs
 	__log_locale_settings('unmodified startup locale settings (should be [C])')
 
 	# activate user-preferred locale
@@ -255,7 +258,6 @@ def activate_locale():
 	except:
 		_log.exception('error activating user-default locale')
 
-	# logging state of affairs
 	__log_locale_settings('locale settings after activating user-default locale')
 
 	# did we find any locale setting ? assume en_EN if not
@@ -269,6 +271,7 @@ def activate_locale():
 	__split_locale_into_levels()
 
 	return True
+
 #---------------------------------------------------------------------------
 def install_domain(domain=None, language=None, prefer_local_catalog=False):
 	"""Install a text domain suitable for the main script."""
@@ -325,7 +328,7 @@ def install_domain(domain=None, language=None, prefer_local_catalog=False):
 	# - $(<script-name>_DIR)/
 	env_key = "%s_DIR" % os.path.splitext(os.path.basename(sys.argv[0]))[0].upper()
 	_log.debug('looking at ${%s}' % env_key)
-	if os.environ.has_key(env_key):
+	if env_key in os.environ:
 		loc_dir = os.path.abspath(os.path.join(os.environ[env_key], 'po'))
 		_log.debug('${%s} = "%s" -> [%s]' % (env_key, os.environ[env_key], loc_dir))
 		candidates.append(loc_dir)
@@ -375,6 +378,7 @@ def install_domain(domain=None, language=None, prefer_local_catalog=False):
 	dummy = gettext.NullTranslations()
 	dummy.install()
 	return True
+
 #===========================================================================
 _encoding_mismatch_already_logged = False
 _current_encoding = None
@@ -419,6 +423,7 @@ def get_encoding():
 		_encoding_mismatch_already_logged = True
 
 	return locale.getpreferredencoding(do_setlocale=False)
+
 #===========================================================================
 # Main
 #---------------------------------------------------------------------------
@@ -431,30 +436,53 @@ if __name__ == "__main__":
 		sys.exit()
 
 	logging.basicConfig(level = logging.DEBUG)
+	#----------------------------------------------------------------------
+	def test_strcoll():
+		candidates = [
+#			(u'a', u'a'),
+#			(u'a', u'b'),
+#			(u'1', u'1'),
+#			(u'1', u'2'),
+#			(u'A', u'A'),
+#			(u'a', u'A'),
+			(u'\u270d', u'\u270d'),
+			(u'4', u'\u270d' + u'4'),
+			(u'4.4', u'\u270d' + u'4.4'),
+			(u'44', u'\u270d' + u'44'),
+			(u'4', u'\u270d' + u'9'),
+			(u'4', u'\u270d' + u'2'),
+#			(u'9', u'\u270d' + u'9'),
+#			(u'9', u'\u270d' + u'4'),
 
-	print "======================================================================"
-	print "GNUmed i18n"
-	print ""
-	print "authors:", __author__
-	print "license:", __license__, "; version:", __version__
-	print "======================================================================"
+		]
+		for cands in candidates:
+			print(cands[0], u'<vs>', cands[1], '=', locale.strcoll(cands[0], cands[1]))
+#			print(cands[1], u'<vs>', cands[0], '=', locale.strcoll(cands[1], cands[0]))
+
+	#----------------------------------------------------------------------
+	print("======================================================================")
+	print("GNUmed i18n")
+	print("")
+	print("authors:", __author__)
+	print("license:", __license__)
+	print("======================================================================")
 
 	activate_locale()
-	print "system locale: ", system_locale, "; levels:", system_locale_level
-	print "likely encoding:", get_encoding()
+	print("system locale: ", system_locale, "; levels:", system_locale_level)
+	print("likely encoding:", get_encoding())
 
 	if len(sys.argv) > 1:
 		install_domain(domain = sys.argv[2])
 	else:
 		install_domain()
-	# ********************************************************
-	# == do not remove this line =============================
-	# it is needed to check for successful installation of
-	# the desired message catalog
-	# ********************************************************
-	tmp = _('Translate this or i18n into <en_EN> will not work properly !')
-	# ********************************************************
-	# ********************************************************
 
-#=====================================================================
+	test_strcoll()
 
+	# ********************************************************************* #
+	# == do not remove this line ========================================== #
+	# it is needed to check for successful installation of                  #
+	# the desired message catalog                                           #
+	# ********************************************************************* #
+	tmp = _('Translate this or i18n into <en_EN> will not work properly !') #
+	# ********************************************************************* #
+	# ********************************************************************* #
