@@ -1,4 +1,7 @@
 # -*- coding: utf-8 -*-
+
+from __future__ import print_function
+
 __doc__ = """GNUmed internetworking tools."""
 
 #===========================================================================
@@ -17,6 +20,7 @@ import mimetools
 import StringIO
 import zipfile
 import webbrowser
+import io
 
 
 # GNUmed libs
@@ -35,6 +39,7 @@ _log = logging.getLogger('gm.net')
 #---------------------------------------------------------------------------
 def open_url_in_browser(url, new=2, autoraise=True, *args, **kwargs):
 	# url, new=0, autoraise=True
+	# new=2: open new tab if possible
 	try:
 		webbrowser.open(url, new = new, autoraise = autoraise, **kwargs)
 	except (webbrowser.Error, OSError, UnicodeEncodeError):
@@ -70,7 +75,7 @@ def download_data_pack(pack_url, filename=None, md5_url=None):
 	_log.debug('downloading MD5 from: %s', md5_url)
 	md5_fname = download_file(md5_url, filename = dp_fname + u'.md5')
 
-	md5_file = open(md5_fname, 'rU')
+	md5_file = io.open(md5_fname, mode = 'rt', encoding = 'utf8')
 	md5_expected = md5_file.readline().strip('\n')
 	md5_file.close()
 	_log.debug('expected MD5: %s', md5_expected)
@@ -428,14 +433,14 @@ Subject: gmTools test suite mail
 
 This is a test mail from the gmTools.py module.
 """ % (default_mail_receiver, default_mail_sender)
-		print "mail sending succeeded:", send_mail (
+		print("mail sending succeeded:", send_mail (
 #			sender = 'abc@xyz.123',
 			receiver = [default_mail_receiver, u'karsten.hilbert@gmx.net'],
 			message = msg,
 			auth = {'user': default_mail_sender, 'password': u'gnumed-at-gmx-net'}, # u'gm/bugs/gmx'
 			debug = True,
 			attachments = [[sys.argv[2]]]
-		)
+		))
 	#-----------------------------------------------------------------------
 	def test_check_for_update():
 
@@ -448,9 +453,9 @@ This is a test mail from the gmTools.py module.
 		]
 
 		for test in test_data:
-			print "arguments:", test
+			print("arguments:", test)
 			found, msg = check_for_update(test[0], test[1], test[2], test[3])
-			print msg
+			print(msg)
 
 		return
 	#-----------------------------------------------------------------------
@@ -459,13 +464,13 @@ This is a test mail from the gmTools.py module.
 		#url = 'missing-file.zip'
 		url = 'gmTools.py'
 		dl_name = download_data_pack(url)
-		print url, "->", dl_name
+		print(url, "->", dl_name)
 		unzip_dir = unzip_data_pack(dl_name)
-		print "unzipped into", unzip_dir
+		print("unzipped into", unzip_dir)
 	#-----------------------------------------------------------------------
 	def test_browser():
 		success = open_url_in_browser(sys.argv[2])
-		print success
+		print(success)
 		open_url_in_browser(sys.argv[2], abc=222)
 	#-----------------------------------------------------------------------
 	#test_check_for_update()
