@@ -46,7 +46,7 @@ _ID_FORM_DISPOSAL_ARCHIVE_ONLY = range(4)
 #============================================================
 # generic form generation and handling convenience functions
 #------------------------------------------------------------
-def print_doc_from_template(parent=None, jobtype=None, episode=None):
+def print_doc_from_template(parent=None, jobtype=None, episode=None, edit_form=None):
 
 	form = generate_form_from_template (
 		parent = parent,
@@ -55,7 +55,7 @@ def print_doc_from_template(parent=None, jobtype=None, episode=None):
 			u'visual progress note',
 			u'invoice'
 		],
-		edit = True
+		edit = edit_form			# default None = respect template setting
 	)
 	if form is None:
 		return False
@@ -63,11 +63,15 @@ def print_doc_from_template(parent=None, jobtype=None, episode=None):
 	if form in [True, False]:	# returned by special OOo/LO handling
 		return form
 
+	if episode is None:
+		epi_name = u'administrative'
+	else:
+		epi_name = episode['description']
 	return act_on_generated_forms (
 		parent = parent,
 		forms = [form],
 		jobtype = jobtype,
-		episode_name = u'administrative',
+		episode_name = epi_name,
 		review_copy_as_normal = True
 	)
 
@@ -240,7 +244,8 @@ def act_on_generated_forms(parent=None, forms=None, jobtype=None, episode_name=N
 				unlock_patient = False,
 				episode = epi,
 				review_as_normal = review_copy_as_normal,
-				reference = None
+				reference = None,
+				pk_org_unit = gmPraxis.gmCurrentPraxisBranch()['pk_org_unit']
 			)
 
 		return True
