@@ -46,6 +46,7 @@ WEB_CHARS = "a-zA-Z0-9\.\-_/:"
 
 
 _timers = []
+
 #============================================================
 def shutdown():
 	"""It can be useful to call this early from your shutdown code to avoid hangs on Notify()."""
@@ -66,6 +67,7 @@ class _cPRWTimer(wx.Timer):
 
 	def Notify(self):
 		self.callback()
+
 #============================================================
 # FIXME: merge with gmListWidgets
 class cPhraseWheelListCtrl(wx.ListCtrl, listmixins.ListCtrlAutoWidthMixin):
@@ -101,6 +103,7 @@ class cPhraseWheelListCtrl(wx.ListCtrl, listmixins.ListCtrlAutoWidthMixin):
 		if sel_idx == -1:
 			return None
 		return self.__data[sel_idx]['list_label']
+
 #============================================================
 # base class for both single- and multi-phrase phrase wheels
 #------------------------------------------------------------
@@ -250,25 +253,46 @@ class cPhraseWheelBase(wx.TextCtrl):
 		raise NotImplementedError('[%s]: set_from_pk()' % self.__class__.__name__)
 	#--------------------------------------------------------
 	def display_as_valid(self, valid=None, partially_invalid=False):
+
 		if valid is True:
-			self.SetBackgroundColour(self.__my_startup_color)
+			color2show = self.__my_startup_color
 		elif valid is False:
 			if partially_invalid:
-				self.SetBackgroundColour(color_prw_partially_invalid)
+				color2show = color_prw_partially_invalid
 			else:
-				self.SetBackgroundColour(color_prw_invalid)
+				color2show = color_prw_invalid
 		else:
 			raise ValueError(u'<valid> must be True or False')
-		self.Refresh()
+
+		if self.IsEnabled():
+			self.SetBackgroundColour(color2show)
+			self.Refresh()
+			return
+
+		self.__previous_enabled_bg_color = color2show
 	#--------------------------------------------------------
-	def display_as_disabled(self, disabled=None):
-		if disabled is True:
+	def Disable(self):
+		self.Enable(enable = False)
+	#--------------------------------------------------------
+	def Enable(self, enable=True):
+		if self.IsEnabled() is enable:
+			return
+
+		if self.IsEnabled():
+			self.__previous_enabled_bg_color = self.GetBackgroundColour()
+
+		super(cPhraseWheelBase, self).Enable(enable)
+
+		if enable is True:
+			#self.SetBackgroundColour(color_prw_valid)
+			self.SetBackgroundColour(self.__previous_enabled_bg_color)
+		elif enable is False:
 			self.SetBackgroundColour(wx.SystemSettings_GetColour(wx.SYS_COLOUR_BACKGROUND))
-		elif disabled is False:
-			self.SetBackgroundColour(color_prw_valid)
 		else:
-			raise ValueError(u'<disabled> must be True or False')
+			raise ValueError(u'<enable> must be True or False')
+
 		self.Refresh()
+
 	#--------------------------------------------------------
 	# callback API
 	#--------------------------------------------------------
@@ -1398,7 +1422,7 @@ if __name__ == '__main__':
 				firstnames || ' ' || lastnames || ', ' || to_char(dob, 'YYYY-MM-DD'),
 				firstnames || ' ' || lastnames
 			from
-				dem.v_basic_person
+				dem.v_active_persons
 			where
 				firstnames || lastnames %(fragment_condition)s
 		"""
